local BaikItemLevel = LibStub("AceAddon-3.0"):NewAddon("BaikItemLevel", "AceConsole-3.0")
local DEBUG = false

---- BaikItemLevel Functions ----
function BaikItemLevel:DPrint(fmt, ...)
    if DEBUG then
        self:Print(string.format(fmt, ...))
    end
end

local DEFAULTS = {
    profile = {
        filterLevel = 150,
        colorLevelMax = 25,
        colorLevelMin  = -35
    }
}

local optCounter = -1
local function nextOrder()
    optCounter = optCounter + 1
    return optCounter
end

local NUMBER_PATTERN = "(-?%d+)"

local options = {
    name = "BaikItemLevel",
    handler = BaikItemLevel,
    type = "group",
    args = {
        general = {
            name = "General Options",
            type = "group",
            args = {
                filter_header = {
                    name = "Item Level Filter",
                    type = "header",
                    order = nextOrder(),
                },
                filter_number = {
                    name = "Item Level",
                    type = "input",
                    pattern = NUMBER_PATTERN,
                    usage = "Default (150)",
                    order = nextOrder(),
                    get = function()
                        return tostring(BaikItemLevel.db.profile.filterLevel)
                    end,
                    set = function(info, value)
                        local number = string.match(value, NUMBER_PATTERN)

                        BaikItemLevel.db.profile.filterLevel = tonumber(number)
                        BaikItemLevel:Update()
                    end
                },
                color_range_header = {
                    name = "ItemLevel Color",
                    type = "header",
                    order = nextOrder(),
                },
                color_level_min = {
                    name = "Min Level",
                    type = "input",
                    pattern = NUMBER_PATTERN,
                    usage = "Default (-35)",
                    order = nextOrder(),
                    get = function()
                        return tostring(BaikItemLevel.db.profile.colorLevelMin)
                    end,
                    set = function(info, value)
                        local number = string.match(value, NUMBER_PATTERN)

                        BaikItemLevel.db.profile.colorLevelMin = tonumber(number)
                        BaikItemLevel:Update()
                    end
                },
                color_level_max = {
                    name = "Max Level",
                    type = "input",
                    pattern = NUMBER_PATTERN,
                    usage = "Default (25)",
                    order = nextOrder(),
                    get = function()
                        return tostring(BaikItemLevel.db.profile.colorLevelMax)
                    end,
                    set = function(info, value)
                        local number = string.match(value, NUMBER_PATTERN)

                        BaikItemLevel.db.profile.colorLevelMax = tonumber(number)
                        BaikItemLevel:Update()
                    end
                },
            }
        }
    }
}

---- BaikItemLevel Functions ----
function BaikItemLevel:Update()
    for _, module in pairs(self.modules) do
        module:OnUpdate()
    end
end

---- Ace Callbacks ----
function BaikItemLevel:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("BaikItemLevelDB", DEFAULTS, true)
    self.options = options
    self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
    self.options.args.profiles["order"] = -1
    self.modules = {
        self:GetModule("Bag"),
        self:GetModule("Inspect"),
        self:GetModule("Inventory")
    }
    self:DPrint("OnInitialize")
end

function BaikItemLevel:OnEnable()
    LibStub("AceConfig-3.0"):RegisterOptionsTable("BaikItemLevel", self.options)
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BaikItemLevel", "BaikItemLevel")
    self:DPrint("OnEnable")
end

function BaikItemLevel:OnDisable()
end
