-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Bag = BaikItemLevel:NewModule("Bag", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local BagFrame = BaikItemLevel.BagFrame
local TableHelper = BaikItemLevel.TableHelper

-- Constants
local FRAMES = BagFrame.FRAMES

function Bag:CheckMissing(bag)
    if bag:HasMissing() and not self.callback then
        self.callback = self:ScheduleTimer("LoadMissing", 5)
    end
end

function Bag:LoadBag(id)
    local bag = self.bags[id]
    if not bag then
        bag = BagFrame.new(id)
        self.bags[id] = bag
    end
    bag:Load()
    self:CheckMissing(bag)
end

function Bag:LoadMissing()
    self.callback = nil
    for _, bag in pairs(self.bags) do
        if bag:HasMissing() then
            bag:UpdateMissing()
            self:CheckMissing(bag)
        end
    end
end

function Bag:Load(frame)
    if self:IsHooked(frame, "OnShow") then
        self:Unhook(frame, "OnShow")
    end
    local id = TableHelper:FindKey(FRAMES, frame)
    if id then
        self:LoadBag(id)
    end
end

function Bag:OnUpdate()
    for id, _ in pairs(FRAMES) do
        self:LoadBag(id)
    end
end

-- Event Callbacks
function Bag:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    for _, bag in pairs(self.bags) do
        bag:UpdateColor()
    end
end

function Bag:BAG_UPDATE(opt, id)
    self.updateBags[id] = true
end

function Bag:BAG_UPDATE_DELAYED()
    local bagCache = TableHelper:Copy(self.updateBags)
    self.updateBags = {}

    for id, _ in pairs(bagCache) do
        local bag = self.bags[id]
        if bag then
            bag:Load()
        end
    end
end

-- Ace3 Callbacks
function Bag:OnInitialize()
    self.bags = {}
    self.updateBags = {}
    BaikItemLevel:DPrint("Bag OnInitialize")
end

function Bag:OnEnable()
    for id, frame in pairs(FRAMES) do
        self:HookScript(frame, "OnShow", "Load")
    end
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    self:RegisterEvent("BAG_UPDATE")
    self:RegisterEvent("BAG_UPDATE_DELAYED")
    BaikItemLevel:DPrint("Bag OnEnable")
end

function Bag:OnDisable()
    BaikItemLevel:DPrint("Bag OnDisable")
end
