-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Inspect = BaikItemLevel:NewModule("Inspect", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local ItemHelper = BaikItemLevel.ItemHelper
local ItemLevelBadge = BaikItemLevel.ItemLevelBadge
local TableHelper = BaikItemLevel.TableHelper

-- Constants
local ROOT_FRAME = nil
local FRAMES = nil

function Inspect:SetSlot(slot)
    local itemLevel, hasItem = ItemHelper:GetPlayerInventorySlotLevel(slot, InspectFrame.unit)
    local badge = self.badges[slot]
    if hasItem then
        -- Cache slot so we can retreive item info when available
        if not itemLevel then
            table.insert(self.missing, slot)
            itemLevel = ItemLevelBadge.INVALID
            if not self.callback then
                self.callback = self:ScheduleTimer("LoadMissing", 5)
            end
        end

        -- Setup UI
        if not badge then
            badge = ItemLevelBadge.new(FRAMES[slot])
            self.badges[slot] = badge
        end
        badge:SetLevel(itemLevel)
    elseif badge then
        -- Item no longer exists
        badge:Hide()
        self.badges[slot] = nil
    end
end

function Inspect:LoadMissing()
    local slotTable = TableHelper:Copy(self.missing)
    self.missing = {}
    self.callback = nil

    for _, slot in pairs(slotTable) do
        self:SetSlot(slot)
    end
end

function Inspect:Load()
    for slot = INVSLOT_FIRST_EQUIPPED, INVSLOT_LAST_EQUIPPED, 1 do
        self:SetSlot(slot)
    end
end

function Inspect:Setup()
    ROOT_FRAME = _G["InspectPaperDollItemsFrame"]
    FRAMES = {
        [INVSLOT_HEAD] = _G["InspectHeadSlot"],
        [INVSLOT_NECK] = _G["InspectNeckSlot"],
        [INVSLOT_SHOULDER] = _G["InspectShoulderSlot"],
        [INVSLOT_BODY] = _G["InspectShirtSlot"],
        [INVSLOT_CHEST] = _G["InspectChestSlot"],
        [INVSLOT_WAIST] = _G["InspectWaistSlot"],
        [INVSLOT_LEGS] = _G["InspectLegsSlot"],
        [INVSLOT_FEET] = _G["InspectFeetSlot"],
        [INVSLOT_WRIST] = _G["InspectWristSlot"],
        [INVSLOT_HAND] = _G["InspectHandsSlot"],
        [INVSLOT_FINGER1] = _G["InspectFinger0Slot"],
        [INVSLOT_FINGER2] = _G["InspectFinger1Slot"],
        [INVSLOT_TRINKET1] = _G["InspectTrinket0Slot"],
        [INVSLOT_TRINKET2] = _G["InspectTrinket1Slot"],
        [INVSLOT_BACK] = _G["InspectBackSlot"],
        [INVSLOT_MAINHAND] = _G["InspectMainHandSlot"],
        [INVSLOT_OFFHAND] = _G["InspectSecondaryHandSlot"],
        [INVSLOT_TABARD] = _G["InspectTabardSlot"]
    }
    self:RegisterEvent("UNIT_INVENTORY_CHANGED")
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    self:RegisterEvent("INSPECT_READY")
end

function Inspect:OnUpdate()
end

-- Event Callbacks
function Inspect:UNIT_INVENTORY_CHANGED(opt, unitId)
    if ROOT_FRAME.unit == unitId then
        self:Load()
    end
end

function Inspect:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    for _, badge in pairs(self.badges) do
        badge:UpdateColor()
    end
end

function Inspect:INSPECT_READY()
    self:Load()
end

function Inspect:ADDON_LOADED()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        self:Setup()
        self:UnregisterEvent("ADDON_LOADED")
    end
end

-- Ace3 Callbacks
function Inspect:OnInitialize()
    self.badges = {}
    self.missing = {}
    BaikItemLevel:DPrint("Inspect OnInitialize")
end

function Inspect:OnEnable()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        self:Setup()
    else
        self:RegisterEvent("ADDON_LOADED")
    end

    BaikItemLevel:DPrint("Inspect OnEnable")
end

function Inspect:OnDisable()
    BaikItemLevel:DPrint("Inspect OnDisable")
end
