-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Inventory = BaikItemLevel:NewModule("Inventory", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local ItemHelper = BaikItemLevel.ItemHelper
local ItemLevelBadge = BaikItemLevel.ItemLevelBadge
local TableHelper = BaikItemLevel.TableHelper

-- Constants
local ROOT_FRAME = PaperDollFrame
local FRAMES = {
    [INVSLOT_HEAD] = CharacterHeadSlot,
    [INVSLOT_NECK] = CharacterNeckSlot,
    [INVSLOT_SHOULDER] = CharacterShoulderSlot,
    [INVSLOT_BODY] = CharacterShirtSlot,
    [INVSLOT_CHEST] = CharacterChestSlot,
    [INVSLOT_WAIST] = CharacterWaistSlot,
    [INVSLOT_LEGS] = CharacterLegsSlot,
    [INVSLOT_FEET] = CharacterFeetSlot,
    [INVSLOT_WRIST] = CharacterWristSlot,
    [INVSLOT_HAND] = CharacterHandsSlot,
    [INVSLOT_FINGER1] = CharacterFinger0Slot,
    [INVSLOT_FINGER2] = CharacterFinger1Slot,
    [INVSLOT_TRINKET1] = CharacterTrinket0Slot,
    [INVSLOT_TRINKET2] = CharacterTrinket1Slot,
    [INVSLOT_BACK] = CharacterBackSlot,
    [INVSLOT_MAINHAND] = CharacterMainHandSlot,
    [INVSLOT_OFFHAND] = CharacterSecondaryHandSlot,
    [INVSLOT_TABARD] = CharacterTabardSlot
}

function Inventory:SetSlot(slot)
    local itemLevel, hasItem = ItemHelper:GetPlayerInventorySlotLevel(slot)
    local badge = self.badges[slot]
    if hasItem then
        -- Cache slot so we can retreive item info when available
        if not itemLevel then
            table.insert(self.missing, slot)
            itemLevel = ItemLevelBadge.INVALID
            if not self.callback then
                self.callback = self:ScheduleTimer("LoadMissing", 5)
            end
        end

        -- Setup UI
        if not badge then
            badge = ItemLevelBadge.new(FRAMES[slot], itemLevel)
            self.badges[slot] = badge
        end
        badge:SetLevel(itemLevel)
    elseif badge then
        -- Item no longer exists
        badge:Hide()
        self.badges[slot] = nil
    end
end

function Inventory:LoadMissing()
    local slotTable = TableHelper:Copy(self.missing)
    self.missing = {}
    self.callback = nil

    for _, slot in pairs(slotTable) do
        self:SetSlot(slot)
    end
end

function Inventory:Load()
    if self:IsHooked(ROOT_FRAME, "OnShow") then
        self:Unhook(ROOT_FRAME, "OnShow")
    end
    for slot = INVSLOT_FIRST_EQUIPPED, INVSLOT_LAST_EQUIPPED, 1 do
        self:SetSlot(slot)
    end
end

function Inventory:OnUpdate()
    self:Load()
end

-- Event Callbacks
function Inventory:PLAYER_EQUIPMENT_CHANGED(opt, slot, hasItem)
    self:SetSlot(slot)
end

function Inventory:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    for _, badge in pairs(self.badges) do
        badge:UpdateColor()
    end
end

-- Ace3 Callbacks
function Inventory:OnInitialize()
    self.badges = {}
    self.missing = {}
    BaikItemLevel:DPrint("Inventory OnInitialize")
end

function Inventory:OnEnable()
    self:HookScript(ROOT_FRAME, "OnShow", "Load")
    self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    BaikItemLevel:DPrint("Inventory OnEnable")
end

function Inventory:OnDisable()
    BaikItemLevel:DPrint("Inventory OnDisable")
end
