local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local ItemHelper = BaikItemLevel.ItemHelper
local ItemLevelBadge = BaikItemLevel.ItemLevelBadge
local TableHelper = BaikItemLevel.TableHelper
local BagFrame = {}

-- Add Module to Addon
BagFrame.__index = BagFrame
BaikItemLevel.BagFrame = BagFrame

-- Constants
BagFrame.FRAMES = {
    [BACKPACK_CONTAINER] = ContainerFrame1,
    ContainerFrame2,
    ContainerFrame3,
    ContainerFrame4,
    ContainerFrame5
}

-- Private Methods
local function GetSlotFrame(container, slot, size)
    local containerId = container + 1
    local slotId = size - slot + 1

    return _G[string.format("ContainerFrame%dItem%d", containerId, slotId)]
end

-- BagFrame API

function BagFrame.new(id)
    local self = setmetatable({}, BagFrame)

    -- Setup frame
    local frameName = string.format("BaikItemLevel_Bag%d", id)
    local frame = CreateFrame("Frame", frameName, self.FRAMES[id])
    frame:SetAllPoints()
    frame:Show()

    -- Assign variables
    self.frame = frame
    self.id = id
    self.size = GetContainerNumSlots(id)
    self.badges = {}
    self.missing = {}

    return self
end

function BagFrame:SetSlot(slot)
    local itemLevel, hasItem = ItemHelper:GetBagSlotLevel(self.id, slot)
    local badge = self.badges[slot]

    if hasItem then
        if not itemLevel then
            table.insert(self.missing, slot)
            itemLevel = ItemLevelBadge.INVALID
        end

        -- Setup UI
        if not badge then
            local anchor = GetSlotFrame(self.id, slot, self.size)
            badge = ItemLevelBadge.new(anchor)
            self.badges[slot] = badge
        end
        badge:SetLevel(itemLevel)
    elseif badge then
        -- Item no longer exists
        badge:Hide()
        self.badges[slot] = nil
    end
end

function BagFrame:UpdateColor()
    for _, badge in pairs(self.badges) do
        badge:UpdateColor()
    end
end

function BagFrame:UpdateMissing()
    local missingCache = TableHelper:Copy(self.missing)
    self.missing = {}
    for _, slot in pairs(missingCache) do
        self:SetSlot(slot)
    end
end

function BagFrame:HasMissing()
    return not TableHelper:IsEmpty(self.missing)
end

function BagFrame:Load()
    local size = GetContainerNumSlots(self.id)
    if size < self.size then
        -- If bag size changes, we want to clean up leftover
        for slot = size, self.size, 1 do
            self.badges[slot]:Hide()
            self.badges[slot] = nil
        end
    end

    for slot = 1, self.size, 1 do
        self:SetSlot(slot)
    end
end
